==> grizzly-framework-2.3.7-sources.jar: org/glassfish/grizzly/threadpool/FixedThreadPool.java <==
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2010-2013 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package org.glassfish.grizzly.threadpool;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.RejectedExecutionException;
import org.glassfish.grizzly.utils.DataStructures;

/**
 * Minimalistic fixed thread pool to allow for nice scalability if a
 * good Queue impl is used.
 *
 * @author gustav trede
 */
public class FixedThreadPool extends AbstractThreadPool {

    protected final BlockingQueue<Runnable> workQueue;

    public FixedThreadPool(ThreadPoolConfig config) {
        super(config);

        this.workQueue = config.getQueue() != null ?
            (BlockingQueue<Runnable>) config.getQueue() :
            (BlockingQueue<Runnable>) config.setQueue(
                DataStructures.getLTQInstance(Runnable.class)).getQueue();
        
        int poolSize = config.getMaxPoolSize();

        synchronized (stateLock) {
            while (poolSize-- > 0) {
                doStartWorker();
            }
        }

        ProbeNotifier.notifyThreadPoolStarted(this);
        super.onMaxNumberOfThreadsReached();
    }

    /**
     * Must hold statelock while calling this method.
     */
    private void doStartWorker() {
        startWorker(new BasicWorker());
    }

    @Override
    public void execute(Runnable command) {
        if (running) {
            if (workQueue.offer(command)) {
                // doublecheck the pool is still running
                if (!running && workQueue.remove(command)) {
                    throw new RejectedExecutionException("ThreadPool is not running");
                }
                
                onTaskQueued(command);
                return;
            }
            onTaskQueueOverflow();
            return;
        }
        throw new RejectedExecutionException("ThreadPool is not running");
    }

    private final class BasicWorker extends Worker {
        @Override
        protected final Runnable getTask() throws InterruptedException {
            return workQueue.take();
        }
    }
}

==> grizzly-framework-2.4.3-sources.jar: org/glassfish/grizzly/threadpool/FixedThreadPool.java <==
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2010-2017 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package org.glassfish.grizzly.threadpool;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedTransferQueue;
import java.util.concurrent.RejectedExecutionException;

/**
 * Minimalistic fixed thread pool to allow for nice scalability if a
 * good Queue impl is used.
 *
 * @author gustav trede
 */
public class FixedThreadPool extends AbstractThreadPool {

    protected final BlockingQueue<Runnable> workQueue;

    public FixedThreadPool(ThreadPoolConfig config) {
        super(config);

        this.workQueue = config.getQueue() != null ?
            (BlockingQueue<Runnable>) config.getQueue() :
            (BlockingQueue<Runnable>) config.setQueue(
                new LinkedTransferQueue<>()).getQueue();
        
        int poolSize = config.getMaxPoolSize();

        synchronized (stateLock) {
            while (poolSize-- > 0) {
                doStartWorker();
            }
        }

        ProbeNotifier.notifyThreadPoolStarted(this);
        super.onMaxNumberOfThreadsReached();
    }

    /**
     * Must hold statelock while calling this method.
     */
    private void doStartWorker() {
        startWorker(new BasicWorker());
    }

    @Override
    public void execute(Runnable command) {
        if (running) {
            if (workQueue.offer(command)) {
                // doublecheck the pool is still running
                if (!running && workQueue.remove(command)) {
                    throw new RejectedExecutionException("ThreadPool is not running");
                }
                
                onTaskQueued(command);
                return;
            }
            onTaskQueueOverflow();
            return;
        }
        throw new RejectedExecutionException("ThreadPool is not running");
    }

    private final class BasicWorker extends Worker {
        @Override
        protected final Runnable getTask() throws InterruptedException {
            return workQueue.take();
        }
    }
}
